from scipy.signal import medfilt
import numpy as np
import matplotlib.pyplot as plt
from types import SimpleNamespace

def clip_outliers(x, y, yerr = None, clip=5, width=15, verbose=True, return_clipped_indices = False):

    """
    Remove outliers using a running median method. Points > clip*M.A.D are removed
    where M.A.D is the mean absolute deviation from the median in each window
    
    Parameters:
    ----------
    x: array_like;
        dependent variable.
        
    y: array_like; same shape as x
        Depedent variable. data on which to perform clipping
        
    yerr: array_like(x);
        errors on the dependent variable
        
    clip: float;
       cut off value above the median. Default is 5
    
    width: int;
        Number of points in window to use when computing the running median. Must be odd. Default is 15
        
    Returns:
    --------
    x_new, y_new, yerr_new: Each and array with the remaining points after clipping
    
    """
    dd = abs( medfilt(y-1, width)+1 - y)   #medfilt pads with zero, so filtering at edge is better if flux level is taken to zero(y-1)
    mad = dd.mean()
    ok= dd < clip * mad

    if verbose:
        print('\nRejected {} points more than {:0.1f} x MAD from the median'.format(sum(~ok),clip))
    
    if yerr is None:
        if return_clipped_indices:
            return x[ok], y[ok], ~ok
            
        return x[ok], y[ok]
    
    if return_clipped_indices:
        return x[ok], y[ok], yerr[ok], ~ok
    
    return x[ok], y[ok], yerr[ok]


def phase_fold(t, period, t0):
    """

    Phasefold data on the give period
    
    Parameters:
    ----------- 
    t: array_like;
        array of times
        
    period: float;
        period
        
    t0: float;
    	reference time
    
    Returns:
    --------
    
    phases: array_like;
        array of phases (not sorted)	
    
    """
    return ((t - t0 + 0.5*period)%period - 0.5*period )/period


def plot_emcee_chains(sampler, labels=None, thin=1, discard=0, figsize=None, alpha=0.05 ):
    """
    Plot chains from emcee sampler run.
    
    Parameters:
    -----------
    sampler: array-like; shape: (nsteps, nwalkers, ndim)
    	Sampler from emcee run
    
    labels: array/list of len ndim
    	Label for the parameters of the chain
    	
    Return:
    -------
    fig
    	
    """
    samples = sampler.get_chain(thin = thin, discard=discard)
    ndim, nwalkers = samples.shape[2], samples.shape[1]
    if figsize is None: figsize = (12,7+int(ndim/2))
    fig, axes = plt.subplots(ndim, sharex=True, figsize=figsize)
    
    if thin > 1 and discard > 0:
        axes[0].set_title(f"Discarded first {discard} steps & thinned by {thin}", fontsize=14)
    elif thin > 1 and discard == 0:
        axes[0].set_title(f"Thinned by {thin}", fontsize=14)
    else:
        axes[0].set_title(f"Discarded first {discard} steps", fontsize=14)
    
    for i in range(ndim):
        ax = axes[i]
        ax.plot(samples[:,:,i],"k", alpha=alpha)
        ax.set_xlim(0,len(samples))
        ax.set_ylabel(labels[i])
    axes[-1].set_xlabel("step number", fontsize=14);
    
    return fig
    
def oversampling(time, oversample_factor, exp_time):

    """
    oversample time of data of long integration time and rebin the data after computation with oversampled time 
    
    Parameters:
    ----------
    
    time : ndarray;
        array of time to oversample
    
    oversampler_factor : int;
        number of points subdividing exposure
    
    exp_time: float;
        exposure time of current data in same units as input time

    Returns:
    --------
    ovs : oversampling object with attributes containing oversampled_time and function to rebin the dependent data back to original cadence.
    
    
    Example:
    --------
    
       t = np.arange(0,1000,10)
       #some function to generate data based on t
       fxn = lambda t: np.random.normal(1,100e-6, len(t))
       #divide each 10min point in t into 30 observations
       ovs = oversampling(t, 30, 10 )
       t_oversampled = ovs.oversampled_time

       #generate value of function at the oversampled time points
       f_ovs = fxn(t_oversampled)
       #then rebin f_ovs back to cadence of observation t
       f = ovs.rebin_data(f_ovs)

    """
    assert isinstance(time, np.ndarray), f'time must be a numpy array and not {type(time)}'
    t_offsets = np.linspace(-exp_time/2., exp_time/2., oversample_factor)
    t_oversample = (t_offsets + time.reshape(time.size, 1)).flatten()
    result = SimpleNamespace(oversampled_time=t_oversample)
    
    def rebin_data(data):
        rebinned_data = np.mean(data.reshape(-1,oversample_factor), axis=1)
        return rebinned_data
    
    result.rebin_data = rebin_data
    
    return result
    





# -*- coding: utf-8 -*-
"""
Created on Tue Nov 27 12:27:41 2018

@author: tunde

Library of functions useful for converting parameters in exoplanet research

TO import from any directory use
>>> import sys; sys.path.append( "/home/tunde/mygit/Planet_parameter_conversions")

"""
import numpy as np

def a_r(P,R,M,format='days'):
    """
    function to convert period to scaled semi-major axis.
    
    Parameters:
    ----------
    P: Period of the planet.
    
    R: Radius of the star in units of solar radii.
    
    M: Mass of star in units of solar masses
    
    format: Unit of P (str). Specify "days" or "years"
    
    
    
    Returns
    -------
    a_r: Scaled semi-major axis.
    
    """
    AU_factor=1.496e8/(R*695510)
    if format=='days':
        P=P/365.
        
    return P**(2/3.)*M**(1/3.)*AU_factor
    

def AU_to_a_r(AU,R):
    """
    function to convert semi-major axis in AU to scaled semi-major axis a/R*.
    
    Parameters:
    ----------
    AU: Semi major axis of the planet in AU.
    
    R: Radius of the star in units of solar radii.
    
    
    Returns
    -------
    a_r: Scaled semi-major axis.
    
    """
    return AU*1.496e8/(R*695510)

    

def impact_parameter(inc,a,format='deg',e=0,w=90):
    """
    Function to convert inclination to impact parameter b.
    input format of angles as 'deg' or 'rad'.
    see eq. 1.19 in https://www.astro.ex.ac.uk/people/alapini/Publications/PhD_chap1.pdf 
    
    Parameters:
    ----------
    inc: inclination of the planetary orbit
    
    a: scaled semi-major axis in units of solar radii    
    
    format: (str) unit of inclincation angle  - "deg" or "rad"
    
    Returns
    -------
    
    b: impact parameter
    """
    import numpy as np

    ecc_factor=(1-e**2)/(1+e*np.sin(np.deg2rad(w)))  
    if format == 'deg':
        inc = np.radians(inc)

    return a*np.cos(inc)*ecc_factor

def inclination(b,a,e=0,w=90):
    """
    Function to convert impact parameter b to inclination in degrees.
    
        
    Parameters:
    ----------
    b: Impact parameter of the transit.
    
    a: Scaled semi-major axis i.e. a/R*
    
    Returns
    --------
    
    inc: The inclination of the planet orbit in degrees.
    
    """
    ecc_factor=(1-e**2)/(1+e*np.sin(np.deg2rad(w)))  
    inc = np.rad2deg(np.arccos( float(b) / (a*ecc_factor)) )
    return round(inc, 2)
    
def vsini(prot,st_rad):
    """
    Function to convert stellar rotation period to vsini in km/s.
    
    
    Parameters:
    ----------
    prot: Rotation period of star in days.
    
    st_rad: Stellar radius in units of solar radii
    
    Returns
    --------
    
    vsini: projected velocity of the star in km/s.
    
    """
    prot=np.array(prot)
    vsini=(2*np.pi*st_rad*696000.)/(prot*24*60*60)
    
    return vsini

def prot(vsini,st_rad):
    """
    Function to convert stellar rotation velocity vsini in km/s to rotation period in days.
    
    
    Parameters:
    ----------
    vsini: Rotation velocity of star in km/s.
    
    st_rad: Stellar radius in units of solar radii
    
    Returns
    ------
    Prot: Period of rotation of the star in days.
    
    """
    vsini=np.array(vsini)

    prot=(2*np.pi*st_rad*696000.)/(vsini*24*60*60)
    
    return prot
    
def kipping_ld(u1,u2):
    """
    Re-parameterize quadratic limb darkening parameters $u_{1}$ and $u_{2}$ according to Kipping (2013)
    
    
    Parameters:
    ----------
    u1: linear limb darkening coefficient.
    
    u2: quadratic limb darkening coefficient.
    
    Returns
    --------
    q1, q2 : Tuple containing the reparametrized limb darkening coefficients
    
    """
    
    q1 = (u1+u2)**2
    q2= u1/(2*(u1+u2))
    
    return round(q1,4), round(q2,4)
				
def planet_prot(f,req,mp,j2):
    """
	Function to calculate period of rotation of a planet
	
	Parameters
	-----------
	
	f : float;
		planet oblateness
	req : float;
		Equatorial radius of planet in jupiter radii.
	mp : float;
		mass of planet in jupiter masses
	j2 : float;
		quadrupole moment of the planet
		
    """
	
    import astropy.constants as c
    import astropy.units as u
    radius=req*c.R_jup
    mass= mp*c.M_jup
    prot=2*np.pi*np.sqrt(radius**3 / (c.G*mass*(2*f-3*j2)))
	
    return prot.to(u.hr).value
    
def ldtk_ldc(lambda_min,lambda_max,Teff,Teff_unc, logg,logg_unc,z,z_unc):
    """
    Function to estimate quadratic limb darkening coefficients for a given star
    
    Parameters:
    ----------
    lambda_min: Start wavelength of the bandpass filter.
    
    lambda_max: End  wavelength of the bandpass filter.

    Teff: Effective temperature of the star.

    Teff_unc: Uncertainty in Teff.

    logg: Surface gravity of the star.

    logg_unc: Uncertainty in logg.

    z: Metallicity of the star.

    z_unc: Uncertainty in z.
    

    Returns
    -------
    cq, eq : Each an array giving the 2 quadractic limb darkening parameters and the errors associated with them 

    
    """
    
    from ldtk import LDPSetCreator, BoxcarFilter
    
    # Define your passbands. Boxcar filters useful in transmission spectroscopy
    filters = [BoxcarFilter('a', lambda_min, lambda_max)] 

    sc = LDPSetCreator(teff=(Teff,   Teff_unc),    # Define your star, and the code
                   logg=(logg, logg_unc),    # downloads the uncached stellar
                      z=(z, z_unc),    # spectra from the Husser et al.
                     filters=filters)    # FTP server automatically.

    ps = sc.create_profiles()                # Create the limb darkening profiles
    cq,eq = ps.coeffs_qd(do_mc=True)         # Estimate quadratic law coefficients

    #lnlike = ps.lnlike_qd([[0.45,0.15],      # Calculate the quadratic law log
#                       [0.35,0.10],      # likelihood for a set of coefficients
#                       [0.25,0.05]])     # (returns the joint likelihood)

    #lnlike = ps.lnlike_qd([0.25,0.05],flt=0) # Quad. law log L for the first filter
    return cq, eq
    

def sigma_CCF(res):
    """
    Function to obtain the CCF width of non-rotating star in km/s based on resolution of spectrograph
    
    
    Parameters:
    ----------
    res: Resolution of spectrograph
    
    Returns
    -------
    sigma: CCF Width of non-rotating star in km/s 
    """
    
    return 3e5/(res*2*np.sqrt(2*np.log(2)))
    
def transit_duration(P,Rp,b,a):
    
    """
    Function to calculate the transit duration
    
    Parameters:
    ----------
    
    P: Period of planet orbit in days

    Rp: Radius of the planet in units of stellar radius

    b: Impact parameter of the planet transit [0, 1+Rp]

    a: scaled semi-major axis of the planet in units of solar radius

    Returns
    -------
    Tdur: duration of transit in hours    
    
    """
    
    tdur= (P*24/np.pi) * (np.arcsin(np.sqrt((1+Rp)**2-b**2)/a))
    
    return  tdur
    
def ingress_duration(P,R,M,Rp,format="days"):
    """
    Function to calculate the duration of ingress/egress.
    
    Parameters:
    ----------
    P: Period of the planet.
    
    R: Radius of the star in units of solar radii.
    
    M: Mass of star in units of solar masses
    
    Rp: Radius of the planet in unit of the stellar radius.
    
    format: Unit of P (str). Specify "days" or "years"
    
    
    
    Returns
    -------
    ingress_dur: Duration of ingres/egress in minutes.       
    
    
    """
    
    if format=='days':
        P=P/365.

    vel= 2*np.pi* a_r(P,R,M,format='years')/float(P)
    
    ingress_dur= 2* Rp/vel  *365*24*60
    
    return ingress_dur
    
def photo_granulation(M,R,Teff):

   """
   Estimate the amplitude and timescale of granulation noise in photometric observations
   as given by Gilliland 2011
   
   Parameters:
   ----------
   
   M: Mass of the star in Solar masses
   
   R: Radius of the star in solar radii
   
   Teff: Effective temperature of the star in Kelvin
   
   Returns
   -------
   
   amp, tau = Amplitude (in ppm) and timescale (in seconds) of the granulation noise in photometry
   
   """
   M, R, Teff= np.array(M), np.array(R), np.array(Teff)
   amp = 75 * M**(-0.5) * R * (Teff/5777.)**0.25  
   
   tau = 220 * M**(-1.) * R**2 * (Teff/5777.)**0.5
   
   return amp, tau
   
def chaplin_exptime(L,Teff,logg):
    """
    Function to compute the optimal exposure time to reduce stellar p-mode oscillation amplitude in the given star to 0.1m/s and 0.09m/s according to Chaplin et al. 2019.
    
    Parameters:
    -----------
    Teff : array_like; size [N]
        An array of stellar effective temperatures in Kelvin.

    logg : array_like; size [N]
        An array of log g's in dex.

    L : array_like; size [N]
        An array of luminosities in units of solar luminosity.

    Returns
    -------
    result : numpy array size [N, 2]
        A 2D array containing tp1 and tpE for each star. tp1 and tpE are exposure times to reach residual amplitudes of 0.1m/s and 0.09m/s to detect earth induced RV. 

    """
    from chaplinfilter import filter
    f = filter(verbose=True)
    results = f(Teff, logg, L)
    return results
    
def rv_precision_degrade(vsini,spec_type):
    """Function to calculate factor by which RV precision of a stellar spectral type degrades due to vsini.
    It compares RV precision at vsini=0km/s to that at *vsini*  in argument of function. It interpolates using quality factor degradation from Table 2 of Bouchy et al.(2001). It fits a 1-D interpolating spline through all the provided data points. 
    
    Parameters:
    ----------
    vsini: vsini of the star
    
    spec_type: Spectral type of the star. Options: "K7V", "K5V", "K2V", "G8V", "F9V", "F5V", "F2V"
    
    Returns
    --------
    Factor by which to multiply the RV precision of the star to get its precision at vsini in argument.
    
    Examples
    --------
    
    For a K2V star whose theoretical RV precision is calculated to be 0.2m/s. This function can be used to estimate the the RV precision at 5km/s.
    
    ::
    
        rv_prec = 0.2
        vsini = 5
        factor=rv_precision_degrade(vsini,K2V)
    
        rv_prec_at_5kms= rv_prec*factor
    
    
    """ 
    import scipy.interpolate as inter    
    
    #table 2 of bouchy et al. (2001)    
    VSINI = [0.,4.,8.,12.,16.,20.]
    
    stars = {'F2V': [14430.0, 9000.0, 4750.0, 2925.0, 2045.0, 1530.0],
             'F5V': [19245.0, 10670.0, 5240.0, 3230.0, 2270.0, 1715.0],
             'F9V': [24450.0, 12685.0, 6105.0, 3785.0, 2690.0, 2045.0],
             'G8V': [30415.0, 14700.0, 7020.0, 4385.0, 3140.0, 2410.0],
             'K2V': [33445.0, 16140.0, 7815.0, 4930.0, 3545.0, 2740.0],
             'K5V': [34940.0, 17080.0, 8440.0, 5380.0, 3885.0, 3020.0],
             'K7V': [31150.0, 15605.0, 8015.0, 5185.0, 3785.0, 2975.0]}    
    
    int_fxn = inter.InterpolatedUnivariateSpline(VSINI,stars[spec_type],k=1)   #
    factor = stars[spec_type][0]/int_fxn(vsini)
    return factor
    
    
def a_r_to_rho_star(P,a_r):
    """
    Compute the transit derived stellar density from the planet period and 
    scaled semi major axis
    
    
    Parameters:
    -----------
    
    P: array-like;
        The planet period in days
    
    a_r: array-like;
        The scaled semi-major axis of the planet orbit
        
    Returns:
    --------
    
    rho: array-like;
        The stellar density in cgs units
        
    """
    import astropy.constants as c
    import astropy.units as u
    
    st_rho=(3*np.pi/(c.G*(P*u.day)**2)*a_r**3).cgs
    return st_rho
    
def rho_to_a_r(rho,P):
    """
    convert stellar density to semi-major axis of planet with a particular period
    
    Parameters:
    -----------
    
    rho: array-like;
        The density of the star in g/cm^3.
        
    P: array-like;
        The period of the planet in days.
        
    Returns:
    --------
    
    a_r: array-like;
        The scaled semi-major axis of the planet.
    """
    
    import astropy.constants as c
    import astropy.units as u
    
    a_r=(((rho*u.g/u.cm**3*(c.G*(P*u.day)**2))/(3*np.pi)).cgs)**(1/3.)
    return a_r.value
    

def T_eq(T_st,a_r):
    """
    calculate equilibrium temperature of planet in Kelvin
    
    Parameters
    ----------
    
    T_st: Array-like;
        Effective Temperature of the star
        
    a_r: Array-like;
        Scaled semi-major axis of the planet orbit
        
    Returns
    -------
    
    T_eq: Array-like;
        Equilibrium temperature of the planet
    """
    print("T_st is {0:.2f}, a_r is {1:.2f}".format(T_st,a_r))
    return T_st*np.sqrt(0.5/a_r)

def R_roche(rho_pl, rho_sat):
    """
    Compute roche radius of a planet as a function of the planet's radius
    
    Parameters:
    ----------
    rho_pl: Array-like;
        density of the planet
        
    rho_sat: Array-like;
        density of the satellite
          
    Returns
    -------
    
    R_roche: Array-like;
        Roche radius of the planet
    """    
    
    return  2.44*((1.0*rho_pl)/rho_sat)**(1/3.)
    
    
def R_hill(mp, m_st, a_r,rp):
    """
    compute the hill radius of a planet
    
    Parameters:
    ----------
    
    mp: array-like;
        mass of the planet in same unit as m_st
        
    m_st: array-like;
        mass of the star
        
    a_r: array-like;
        scaled semi-major axis
    
    rp: array_like;
        ratio of planetary radius to stellar radius
        
    Returns
    --------
    
    R-hill: array-like;
        radius of hill sphere in unit of planetary radius
    
    """
    return ( mp/(3*m_st) )**(1/3.) * a_r/rp
    
def RL_Rroche(j2,mp_ms,a_r,rp_rj,rho_r):
    return 0.75*(j2/0.01)**(1/5.)*(mp_ms/0.001)**(-2/15.)*(rp_rj)**(2/5.)*(a_r/21.5)**(3/5.)*(rho_r/3)**(1/3.)    
    

def phase_fold(time, t0, P):
    """
    Given the observation time and period, return the phase of each observation time
    
    Parameters:
    ----------
    
    time: array-like;
        time of observation

    t0: array-like;
        reference time
        
    P: float;
        Period to use in folding in same unit as t
        
    Returns
    --------
    phases: aaray-like;
        phases of the observation    
    
    """
    
    
    return ( (time-t0) % P) / float(P)
    

def Merge_Pdfs(input_filelist, output_filename):
	"""
	Function to merge pdf documents into a single pdf with different pages
	
	Parameters:
	-----------
	
	input_filelist : list of strings;
		list of path to pdf files with extensions
		
	output_filename : string;
		path of output file to be created with pdf extension
	
	"""
	from PyPDF2 import PdfFileReader, PdfFileWriter
	output = PdfFileWriter()
	for pdf_file in input_filelist:
		pdf_doc = PdfFileReader(open(pdf_file, "rb"))
		[output.addPage(pdf_doc.getPage(i) ) for i in range(pdf_doc.numPages)]
	outputStream = open(output_filename,"wb")
	output.write(outputStream)
	outputStream.close()
	
	



if __name__ == "__main__":    #used to make files usable as a script. so when called from cmd line only codes in this function are excuted. the codes usually calls to the functions above. e.g 
    import sys
    T_eq(np.float(sys.argv[1]), np.float(sys.argv[2]))


import numpy as np
import astropy.constants as c
import astropy.units as u
from uncertainties.umath import  asin, sqrt, log, radians, sin, cos



def effective_ringed_planet_radius(rp,rin,rout,ir):
    """
    Calculate effective radius of a ringed planet accounting for possible overlap between ring and planet. - eqn (2) from zuluaga+2015 http://dx.doi.org/10.1088/2041-8205/803/1/L14
    
    Parameters:
    -----------
    rp : float, ufloat;
        Radius of the planet hosting the ring
    
    rin, rout : float, ufloat;
        Inner and outer radii of the ring in units of rp.
        
    ir : float, ufloat;
        Inclination of the ring from the skyplane. 0 is face-on ring, 90 is edge on
    
    Returns:
    --------
    eff_R : float, ufloat;
        effective radius of the ringed planet in same unit as rp
    	
    """	
    
    cosir = cos(radians(ir))
    sinir = sin(radians(ir))
    y = lambda r: sqrt(r**2 -1)/(r*sinir)

    def eff(r):
        if r*cosir > 1:
            eff_r = r**2 * cosir - 1
        else:
            eff_r = (r**2*cosir * 2/np.pi*asin(y(r))) - (2/np.pi*asin(y(r)*r*cosir))
        
        return eff_r
    rout_eff2 = eff(rout)
    rin_eff2 = eff(rin) 
    
    Arp = rp**2 + rp**2*(rout_eff2 - rin_eff2)
    return sqrt(Arp)
    
    
def R_roche(rho_pl, rho_sat):
    """
    Compute roche radius of a planet as a function of the planet's radius
    
    Parameters:
    ----------
    rho_pl: Array-like;
        density of the planet
        
    rho_sat: Array-like;
        density of the satellite
          
    Returns
    -------
    
    R_roche: Array-like;
        Roche radius of the planet
    """    
    
    return  2.46*((1.0*rho_pl)/rho_sat)**(1/3.)
    
 
def max_ring_density(rho_pl, f=1, verbose=True):
    """
    Use Roche formular to compute maximum density of ring material given inferred planet density in g/cm3.
    since rings cannot be sustained beyond roche limit, we assume the observed radius is the roche limit of possible ringed planet. 
    
    Parameters:
    ----------
    rho_pl: ufloat, float, array-like;
        Inferred density of the planet given the observed radius and estimated mass. ufloat from uncertainties package allows the uncertainties in planet density to be propagated
        to the resulting ring density 
    
    f: float;
    	factor to account for 

    Returns
    -------
    
    rho_ring: Array-like;
        maximum ring density a planet can have. 
    """    
    rho_r = rho_pl * (2.46* f**0.5)**3 
    if verbose: print(f"Any ring around this planet must have density below {rho_r} in order to be within the Roche limit. Check ```T_eq``` of this planet to ensure that ring of such density can exist.")
    return  rho_r
    
    
def ring_density(Rout, rho_pl):
    """
    Calculate the density of the rings material around a ringed planet given the outer ring radius and the density of the host planet.
    
    Parameters:
    ------------
    
    Rout: float, ufloat, array-like;
    	outer ring radius in units of the planet radius Rp.
    	
    rho_pl: float, ufloat, array-like;
    	Density of the host planet.
    	
    Returns:
    --------
    
    rho_r: 
    	ring material density in same unit as rho_pl
    
    """
    
    return (2.46/Rout)**3 * rho_pl
    
    
    
    
def R_hill(mp, m_st, a_r,rp):
    """
    compute the hill radius of a planet
    
    Parameters:
    ----------
    
    mp: array-like;
        mass of the planet in same unit as m_st
        
    m_st: array-like;
        mass of the star
        
    a_r: array-like;
        scaled semi-major axis
    
    rp: array_like;
        ratio of planetary radius to stellar radius
        
    Returns
    --------
    
    R-hill: array-like;
        radius of hill sphere in unit of planetary radius
    
    """
    return ( mp/(3*m_st) )**(1/3.) * a_r/rp
    
def RL_Rroche(j2,mp_ms,a_r,rp_rj,rho_r):
    """
    Calculate the ratio of the laplace radius RL to the Roche radius RR to determine the ring plane.
    The ring plane aligns with the equatorial plane of the planet if RL/RR < 1 since the entirety of the ring will be within the laplace radius.
    RL/RR < 1 implies that the rings extend beyond the laplace radius and thus will transition to lying in the orbital plane of the planet.
    Ref. Schlichting & Chang 2011 - https://iopscience.iop.org/article/10.1088/0004-637X/734/2/117
    
    Parameters:
    -----------
    
    j2: float;
        quadrupole moment of the planet (ranges from∼0.003 for Uranus and Neptune to∼0.01 for Jupiter and Sat-urn (Carter & Winn 2010)) 
    
    mp_ms: float;
        Ratio of the planet-stellar mass.
        
    a_r: float;
        Scaled semi-major axis(unit of stellar radius)
    
    rp_rj: float;
        planet radius in unit of jupiter radii
        
    rho_r: float;
        density of ring particles in g/cm3.
        
    Return:
    -------
    
    RL/RR : float;
        Ratio of Laplace radius to roche radius
    """
    return 0.75*(j2/0.01)**(1/5.)*(mp_ms/0.001)**(-2/15.)*(rp_rj)**(2/5.)*(a_r/21.5)**(3/5.)*(rho_r/3)**(1/3.)    
    



from uncertainties.umath import acos, radians, degrees, sin, cos
import astropy.constants as c
import astropy.units as u
import numpy as np

rsun = c.R_sun.to( u.km).value

def P_to_aR(P,R,M,format='days'):
    """
    function to convert period to scaled semi-major axis.
    
    Parameters:
    ----------
    P: Period of the planet.
    
    R: Radius of the star in units of solar radii.
    
    M: Mass of star in units of solar masses
    
    format: Unit of P (str). Specify "days" or "years"
    
    
    
    Returns
    -------
    aR: Scaled semi-major axis.
    
    """
    AU_factor = c.au.to(u.km).value/(R*rsun)
    if format == 'days':
        P=P/365.
        
    return P**(2/3.)*M**(1/3.)*AU_factor
    

def AU_to_aR(AU,R):
    """
    function to convert semi-major axis in AU to scaled semi-major axis a/R*.
    
    Parameters:
    ----------
    AU: Semi major axis of the planet in AU.
    
    R: Radius of the star in units of solar radii.
    
    
    Returns
    -------
    aR: Scaled semi-major axis.
    
    """
    return AU*c.au.to(u.km).value / (R*rsun)

    

def impact_parameter(inc, a, e=0, w=90, format='deg'):
    """
    Function to convert inclination to impact parameter b.
    input format of angles as 'deg' or 'rad'.
    see eq. 1.19 in https://www.astro.ex.ac.uk/people/alapini/Publications/PhD_chap1.pdf 
    
    Parameters:
    ----------
    inc: float;
        inclination of the planetary orbit
    
    a: float;
        scaled semi-major axis in units of solar radii 
    
    e: float;
        eccentricity of the orbit.
    
    w: float;
        longitude of periastron
      
    
    format: str;  - "deg" or "rad"
        unit of the `inc` and `w`  
    
    Returns
    -------
    
    b: impact parameter
    """

    if format == 'deg':
        inc = radians(inc)
        w = radians(w)

    ecc_factor=(1-e**2)/(1+e*sin(w))  
    return a*cos(inc)*ecc_factor

def inclination(b, a, e=0, w=90):
    """
    Function to convert impact parameter b to inclination in degrees.
    
        
    Parameters:
    ----------
    b: Impact parameter of the transit.
    
    a: Scaled semi-major axis i.e. a/R*.

    e: float;
        eccentricity of the orbit.
    
    w: float;
        longitude of periastron in degrees
    
    Returns
    --------
    
    inc: The inclination of the planet orbit in degrees.
    
    """
    ecc_factor=(1-e**2)/(1+e*sin(radians(w)))  
    inc = degrees(acos( float(b) / (a*ecc_factor)) )
    return round(inc, 2)
    
def vsini(prot, st_rad):
    """
    Function to convert stellar rotation period to vsini in km/s.
    
    
    Parameters:
    ----------
    prot: Rotation period of star in days.
    
    st_rad: Stellar radius in units of solar radii
    
    Returns
    --------
    
    vsini: projected velocity of the star in km/s.
    
    """
    import numpy as np
    prot=np.array(prot)
    vsini=(2*np.pi*st_rad*rsun)/(prot*24*60*60)
    
    return vsini

def prot(vsini, st_rad):
    """
    Function to convert stellar rotation velocity vsini in km/s to rotation period in days.
    
    
    Parameters:
    ----------
    vsini: Rotation velocity of star in km/s.
    
    st_rad: Stellar radius in units of solar radii
    
    Returns
    ------
    Prot: Period of rotation of the star in days.
    
    """
    import numpy as np
    vsini=np.array(vsini)

    prot=(2*np.pi*st_rad*rsun)/(vsini*24*60*60)
    
    return prot
    
    
def convert_LD_coeffs(c1, c2, convert_from = "q2u", verify=True):
    """
    Convert limb darkening (LD) coefficients (c1,c2) from one LD to another.
    
    Parameters:
    -----------
    	
    c1, c2 : float, ufloat, np.array;
        Limb darkening coefficients to convert.
    
    convert_from : string;
        specify which laws to convert from and to.
        - To convert from u1,u2 to q1,q2 following [1] set convert_from = "u2q".
        - To convert from q1,q2 to u1,u2 following [1] set convert_from = "q2u".
        - From h1,h2 to q1,q2 following [1,2] set convert_from = "h2q".
        - From q1,q2 to h1,h2 following [1,2,3] set convert_from = "q2h".	
        - From h1,h2 to u1,u2 following [2,3] set convert_from = "h2u".
        - From u1,u2 to h1,h2 following [2,3] set convert_from = "u2h".
    Verify: bool;
    	confirm that the result satisfies the conditions of the LD law as given in Notes from [1,2,3]
        
    Returns
    -------
    l1, l2 : Tuple;
    	containing the required reparamterized limb darkening coefficients. if input c1, c2 are numpy arrays then l1 and l2 are arrays. 
    
    Note:
    -------
    Conditions: h1>0,  h2>0,  h2 <= h1, and h1<1.
    Conditions: 0< q1<1, 0<=q2<1	
    Conditions: u1+u2<1, u1>0, u1+2u2>0.
    
    References
    ----------
    [1] https://ui.adsabs.harvard.edu/abs/2013MNRAS.435.2152K/abstract    
    [2] https://ui.adsabs.harvard.edu/abs/2018A%26A...616A..39M/abstract
    [3] https://ui.adsabs.harvard.edu/abs/2019RNAAS...3..117S/abstract
           
    """
    assert convert_from in ['u2q','q2u','h2u','u2h','h2q','q2h'], "invalid input for convert_from. Valid options are 'u2q','q2u','h2u','u2h','h2q','q2h' "
    
    if convert_from == "u2q":
        q1 = (c1+c2)**2
        q2 = c1/(2*(c1+c2))
        if verify: assert np.all(0<q1) and np.all(q1<1) and np.all(0<=q2) and np.all(q2<1), f"obtained values of q1={q1} and q2={q2} do not meet the conditions of [1]. See Notes"         
        l1, l2 = q1, q2
        
    elif convert_from == "q2u":
        u1 = 2 *c1**0.5 *c2 
        u2 = c1**0.5*(1-2*c2)
        if verify: assert np.all((u1+u2)<1) and np.all(u1>0) and np.all((u1+2*u2)>0), f"obtained value of u1={u1} and u2={u2} do not meet the conditions. See Notes" 
        l1, l2 = u1, u2
        
    elif convert_from == "h2q": 
        q1 = (1-c2)**2
        q2= (c1-c2)/(1-c2)
        if verify: assert np.all(0<q1) and np.all(q1<1) and np.all(0<=q2) and np.all(q2<1), f"obtained values of q1={q1} and q2={q2} do not meet the conditions of [1]. See Notes"         
        l1, l2 = q1, q2
                   
    elif convert_from == "q2h": 
        h1 = 1 - c1**0.5 + c2*c1**0.5 
        h2 = 1 - c1**0.5
        if verify: assert np.all(0<h1) and np.all(h1<1) and np.all(0<h2) and np.all(h2<=h1), f"obtained values of h1={h1} and h2={h2} do not meet the conditions of [3]. See Notes"
        l1, l2 = h1, h2
        
    elif convert_from == "h2u":
        #h2q
        q1 = (1-c2)**2
        q2= (c1-c2)/(1-c2)
        #q2u
        u1 = 2 *q1**0.5 *q2 
        u2 = q1**0.5*(1-2*q2) 
        if verify: assert np.all((u1+u2)<1) and np.all(u1>0) and np.all((u1+2*u2)>0), f"obtained value of u1={u1} and u2={u2} do not meet the conditions. See Notes" 
        l1, l2 = u1, u2 
         
    elif convert_from == "u2h":
        #u2q
        q1 = (c1+c2)**2
        q2 = c1/(2*(c1+c2)) 
        #q2h
        h1 = 1 - q1**0.5 + q2*q1**0.5 
        h2 = 1 - q1**0.5
        if verify: assert np.all(0<h1) and np.all(h1<1) and np.all(0<h2) and np.all(h2<=h1), f"obtained values of h1={h1} and h2={h2} do not meet the conditions of [3]. See Notes"
        l1, l2 = h1, h2
        
    return l1,l2
    
    
def aR_to_rho_star(P,aR):
    """
    Compute the transit derived stellar density from the planet period and 
    scaled semi major axis
    
    
    Parameters:
    -----------
    
    P: float, ufloat, array-like;
        The planet period in days
    
    aR: float, ufloat, array-like;
        The scaled semi-major axis of the planet orbit
        
    Returns:
    --------
    
    rho: array-like;
        The stellar density in cgs units
        
    """
    import astropy.constants as c
    import astropy.units as u
    import numpy as np
    G = (c.G.to(u.cm**3/(u.g*u.second**2))).value
    Ps = P*(u.day.to(u.second))
    
    st_rho=3*np.pi*aR**3 / (G*Ps**2) 
    return st_rho
    
def rho_to_aR(rho,P):
    """
    convert stellar density to semi-major axis of planet with a particular period
    
    Parameters:
    -----------
    
    rho: float, ufloat, array-like;
        The density of the star in g/cm^3.
        
    P: float, ufloat, array-like;
        The period of the planet in days.
        
    Returns:
    --------
    
    aR: array-like;
        The scaled semi-major axis of the planet.
    """
    
    G = (c.G.to(u.cm**3/(u.g*u.second**2))).value
    Ps = P*(u.day.to(u.second))
    aR = ((rho*G*Ps**2)/(3*np.pi)) **(1/3.)
    return aR
    
    
   

def timeperi_to_timetrans(tp, per, ecc, omega, secondary=False):
    """
    Convert Time of Periastron passage `tp` to Time of Transit i.e time of conjunction `tc`.
    For circular orbits with ecc=0 and w=90. tc = tp.
    Adopted from radvel: https://github.com/California-Planet-Search/radvel
    see also: http://www.sternwarte.uni-erlangen.de/~hanke/science/VelaX-1/ellipticalOrbits.pdf

    
    Parameters:
    -----------
    tp: float;
        time of periastron passage
        
    per: float;
        Planet period in same unit as tp. if tp is a phase then per = 1.
        
    ecc: float;
        eccentricity
        
    omega (float): argument of periastron (degrees)
        
    secondary: bool; 
        calculate time of secondary eclipse instead
    
    Returns:
    -------
    
    tc: float; 
        time of inferior conjunction (time of transit if system is transiting)
    """
    try:
        if ecc >= 1:
            return tp
    except ValueError:
        pass

    if secondary:
        f = 3*np.pi/2 - omega*np.pi/180.                                       # true anomaly during secondary eclipse
        ee = 2 * np.arctan(np.tan(f/2) * np.sqrt((1-ecc)/(1+ecc)))  # eccentric anomaly

        # ensure that ee is between 0 and 2*pi (always the eclipse AFTER tp)
        if isinstance(ee, np.float64):
            ee = ee + 2 * np.pi
        else:
            ee[ee < 0.0] = ee + 2 * np.pi
    else:
        #time of conjunction is time when true anomaly is pi/2. - omega
        f = np.pi/2 - omega*np.pi/180                       
        ee = 2 * np.arctan(np.tan(f/2) * np.sqrt((1-ecc)/(1+ecc)))  # eccentric anomaly

    tc = tp + per/(2*np.pi) * (ee - ecc*np.sin(ee))         # time of conjunction

    return tc


def timetrans_to_timeperi(tc, per, ecc, omega):
    """
    Convert Time of Transit (conjunction/midtransit) to Time of Periastron Passage (Taken from radvel)
    https://github.com/California-Planet-Search/radvel
    
    Parameters:
    ----------
    tc: float;
        time of midtransit (conjunction)
    
    per: float;
        period in same unit as tc. if tc is a phase, use per = 1
    
    ecc: float;
        eccentricity
    
    omega: float;
        longitude of periastron (degrees)
    
    Returns:
    --------
    float: time of periastron passage
    
    """
    try:
        if ecc >= 1:
            return tc
    except ValueError:
        pass

    f = np.pi/2 - omega*np.pi/180
    ee = 2 * np.arctan(np.tan(f/2) * np.sqrt((1-ecc)/(1+ecc)))  # eccentric anomaly
    tp = tc - per/(2*np.pi) * (ee - ecc*np.sin(ee))      # time of periastron

    return tp

def r1r2_to_bp(r1,r2,pl=0.01, pu=0.25):
    """
    Convert uniform samling of r1 and r2 to impact parameter b and and radius ratio p
    following Espinoza 2018, https://iopscience.iop.org/article/10.3847/2515-5172/aaef38/meta
    
    Paramters:
    -----------
    r1, r2: float;
        uniform parameters in from u(0,1)
    
    pl, pu: float;
        lower and upper limits of the radius ratio
        
    
    Return:
    -------
    b, p: tuple;
        impact parameter and radius ratio
    """
    
    assert np.all(0<r1) and np.all(r1<=1) and np.all(0<r2) and np.all(r2<=1), f"r1 and r2 needs to be u(0,1) but r1={r1}, r2={r2}"
    
    Ar = (pu-pl)/(2+pu+pl)
    
    if np.all(r1 > Ar):
        b = (1+pl) * (1 + (r1-1)/(1-Ar) )
        p = (1-r2)*pl + r2*pu
    
    elif np.all(r1 <= Ar):
        q1 = r1/Ar
        
        b = (1+pl) + q1**0.5 * r2*(pu-pl)
        p = pu + (pl-pu)* q1**0.5*(1-r2)
    return b, p
    
    



# -*- coding: utf-8 -*-
"""
Created on Fri Oct 11 14:55:20 2019

@author: tunde
"""
import numpy as np
import warnings

def bic(log_like, n, k):

    """
    Compute the bayesian information criteria
    
    Parameters:
    -----------
    
    log_like: array-like;
        The maximized value of the likelihood function of the model M.
        
    n: array-like;
        Number of data points in the observation.
        
    k: array-like;
        Number of parameters estimated by the model
        
    Returns:
    --------
    
    BIC: array-like input;
        The value of the Bayesian information Criterion. Model with lowest bic is preferred
        
        
    """
    import numpy as np
    return -2. * log_like + k * np.log(n)
    
    
def aic(log_like, n, k):
    """
    The Aikake information criterion.
    A model comparison tool based of infomormation theory. It assumes that N is large i.e.,
    that the model is approaching the CLT.
    
    Parameters:
    -----------
    
    log_like: array-like;
        The maximized value of the likelihood function of the model M.
        
    n: array-like;
        Number of data points in the observation.
        
    k: array-like;
        Number of parameters estimated by the model
        
    Returns:
    --------
    
    AIC: array-like input;
        The value of the Akaike Information Criterion. Model with lowest aic is preferred
    """

    val = -2. * log_like + 2 * k
    val += 2 * k * (k + 1) / float(n - k - 1)

    if not np.isfinite(val):
        val = 0

        warnings.warn('AIC was NAN. Recording zero, but you should examine your fit.')


    return val
    

def rmse(error):
    """
    Calculate the root-mean-square of the imputed error array
    
    Parameters:
    -----------
    
    error : array-like;
            array of error or residual values (observed - calculated)

    Return:
    --------
    rmse : float;
            root mean square error value

    """
    
    return np.sqrt(np.mean(np.square(error)))
    
def sse(error):
    
    return np.sum( np.square(error) )
				
def mse(error):
    """
    Calculate the mean-square error of imputed error array

	Parameters:
    ----------
    error : array-like;
            array of error or residual values (observed - calculated)

    Return:
    --------
    mse : float;
            mean square error value
    """        
    return np.mean(np.square(error))
    
def mae(error):
    """
    Calculate the mean-absolute error of imputed error array

	Parameters:
    ----------
    error : array-like;
            array of error or residual values (observed - calculated)

    Return:
    --------
    mae : float;
            mean absolute error value
    """        
    return np.mean(np.abs(error))
    
def rse(obs, calc):
    """
    Calculate the relative-square error from the observed and calculated values

	Parameters:
    ----------
    obs : array-like;
            array of observed values
    calc : array-like;
            array of calculated values e.g from model
    Return:
    --------
    rse : float;
            relative square error value
    """        
    return np.mean(np.square((obs-calc)/obs))
    
def rae(obs, calc):
    """
    Calculate the relative-absolute error from the observed and calculated values

	Parameters:
    ----------
    obs : array-like;
            array of observed values
    calc : array-like;
            array of calculated values e.g from model
    Return:
    --------
    rae : float;
            relative absolute error value
    """        
    return np.mean(np.abs((obs-calc)/obs))
    
def r_squared(obs, calc):
    """
    Calculate the R2_score commonly referred to as coefficient of determination. It measure how close the regression line is to the  observed values. Best possible value is 1.0
    
	Parameters:
    ----------
    obs : array-like;
            array of observed values
    calc : array-like;
            array of calculated values e.g from model
    Return:
    --------
    r2 : float;
            r2_score value
    """      
  
    return 1 - (np.sum(np.square(obs-calc)) / np.sum(np.square(obs-np.mean(obs))) )


import numpy as np
import astropy.constants as c
import astropy.units as u
from uncertainties.umath import  asin, sqrt, log, radians, sin, cos
from .convert_param import P_to_aR, inclination
			
def planet_prot(f,req,mp,j2):
    """
	Function to calculate period of rotation of a planet
	
	Parameters
	-----------
	
	f : float;
		planet oblateness
	req : float;
		Equatorial radius of planet in jupiter radii.
	mp : float;
		mass of planet in jupiter masses
	j2 : float;
		quadrupole moment of the planet
		
    Returns:
    --------
    prot: rotation peropd of the planet in hours
		
    """
	

    radius=req*c.R_jup
    mass= mp*c.M_jup
    prot=2*np.pi*sqrt(radius**3 / (c.G*mass*(2*f-3*j2)))
	
    return prot.to(u.hr).value
    
    
def transit_prob(Rp, aR, e=0, w=90):
    """
	Function to calculate period of rotation of a planet
	
	Parameters
	-----------
    Rp: float;
        radius of the planet in unit f the stellar radius
    
    aR: float;
        Scaled semi-major axis i.e. a/R*.

    e: float;
        eccentricity of the orbit.
    
    w: float;
        longitude of periastron in degrees
    
		
    """
	#eqn 5 - Kane & von Braun 2008, https://core.ac.uk/reader/216127860
    prob = (1 + Rp)/aR * (1 + e*sin(radians(w))/(1-e**2)  )

	
    return prob
    
    
def ldtk_ldc(lambda_min,lambda_max,Teff,Teff_unc, logg,logg_unc,z,z_unc):
    """
    Function to estimate quadratic limb darkening coefficients for a given star
    
    Parameters:
    ----------
    lambda_min: Start wavelength of the bandpass filter.
    
    lambda_max: End  wavelength of the bandpass filter.

    Teff: Effective temperature of the star.

    Teff_unc: Uncertainty in Teff.

    logg: Surface gravity of the star.

    logg_unc: Uncertainty in logg.

    z: Metallicity of the star.

    z_unc: Uncertainty in z.
    

    Returns
    -------
    cq, eq : Each an array giving the 2 quadractic limb darkening parameters and the errors associated with them 

    
    """
    
    from ldtk import LDPSetCreator, BoxcarFilter
    
    # Define your passbands. Boxcar filters useful in transmission spectroscopy
    filters = [BoxcarFilter('a', lambda_min, lambda_max)] 

    sc = LDPSetCreator(teff=(Teff,   Teff_unc),    # Define your star, and the code
                   logg=(logg, logg_unc),    # downloads the uncached stellar
                      z=(z, z_unc),    # spectra from the Husser et al.
                     filters=filters)    # FTP server automatically.

    ps = sc.create_profiles()                # Create the limb darkening profiles
    cq,eq = ps.coeffs_qd(do_mc=True)         # Estimate quadratic law coefficients

    #lnlike = ps.lnlike_qd([[0.45,0.15],      # Calculate the quadratic law log
#                       [0.35,0.10],      # likelihood for a set of coefficients
#                       [0.25,0.05]])     # (returns the joint likelihood)

    #lnlike = ps.lnlike_qd([0.25,0.05],flt=0) # Quad. law log L for the first filter
    return cq, eq
    

def sigma_CCF(res):
    """
    Function to obtain the CCF width of non-rotating star in km/s based on resolution of spectrograph
    
    
    Parameters:
    ----------
    res: Resolution of spectrograph
    
    Returns
    -------
    sigma: CCF Width of non-rotating star in km/s 
    """
    
    return 3e5/(res*2*sqrt(2*log(2)))
    
def transit_duration(P, Rp, a, b=0, e=0, w=90, inc=None, total=True):
    
    """
    Function to calculate the total (T14) or full (T23) transit duration
    
    Parameters:
    ----------
    
    P: Period of planet orbit in days

    Rp: Radius of the planet in units of stellar radius

    b: Impact parameter of the planet transit [0, 1+Rp]

    a: scaled semi-major axis of the planet in units of solar radius
    
    inc: inclination of the orbit. Optional. If given b is not used. if None, b is used to calculate inc
    
    total: if True calculate the the total transit duration T14, else calculate duration of full transit T23

    Returns
    -------
    Tdur: duration of transit in hours    
    
    """
    #eqn 30 and 31 of Kipping 2010 https://doi.org/10.1111/j.1365-2966.2010.16894.x
        
    factor = (1-e**2)/(1+e*sin(radians(w)))
    if inc == None:
        inc = inclination(b,a,e,w)
    
    if total is False:
        Rp = -Rp
        
    sini = sin(radians(inc))
    cosi = cos(radians(inc))
    
    denom = a*factor*sini
    
    tdur= (P*24/np.pi) * (factor**2/sqrt(1-e**2)) * (asin ( sqrt((1+Rp)**2 - (a*factor*cosi)**2)/ denom ) )
    
    return  tdur
    
def ingress_duration_appox(P, Rp, aR):
    """
    Function to calculate the duration of ingress/egress.
    
    Parameters:
    ---------
    P: Orbital period of the planet in days    
    
    Rp: Radius of the planet in unit of the stellar radius.

    aR: Scaled semi-major axis of the orbit
    
    
    Returns
    -------
    ingress_dur: very rough approximate of the Duration of ingres/egress in minutes.       
    
    
    """
        
    vel= 2*np.pi* aR/P
    
    ingress_dur= 2* Rp/vel  *24*60
    
    return ingress_dur
    
def ingress_duration(P, Rp, a, b=0, e=0, w=90, inc=None, total=True):
    
    """
    Function to calculate the ingress/egress duration T12/23 assuming symmetric light curve
    
    Parameters:
    ----------
    
    P: Period of planet orbit in days

    Rp: Radius of the planet in units of stellar radius

    b: Impact parameter of the planet transit [0, 1+Rp]

    a: scaled semi-major axis of the planet in units of solar radius
    
    inc: inclination of the orbit. Optional. If given b is not used. if None, b is used to calculate inc
    
    total: if True calculate the the total transit duration T14, else calculate duration of full transit T23

    Returns
    -------
    Tdur: duration of transit in hours    
    
    """
    #eqn 43 of Kipping 2010 https://doi.org/10.1111/j.1365-2966.2010.16894.x
        
    T14 = transit_duration(P, Rp, a, b=0, e=e, w=w, inc=inc, total=True)
    T23 = transit_duration(P, Rp, a, b=0, e=e, w=w, inc=inc, total=False)
    
    return  (T14 - T23)/2.

def T_eq(T_st,a_r):
    """
    calculate equilibrium temperature of planet in Kelvin
    
    Parameters
    ----------
    
    T_st: Array-like;
        Effective Temperature of the star
        
    a_r: Array-like;
        Scaled semi-major axis of the planet orbit
        
    Returns
    -------
    
    T_eq: Array-like;
        Equilibrium temperature of the planet
    """
    print("T_st is {0:.2f}, a_r is {1:.2f}".format(T_st,a_r))
    return T_st*sqrt(0.5/a_r)


def phase_fold(time, t0, P):
    """
    Given the observation time and period, return the phase of each observation time
    
    Parameters:
    ----------
    
    time: array-like;
        time of observation

    t0: array-like;
        reference time
        
    P: float;
        Period to use in folding in same unit as t
        
    Returns
    --------
    phases: aaray-like;
        phases of the observation    
    
    """
    
    
    return ( (time-t0) % P) / float(P)
    


#def true_anomaly(t, tp, per, e):
    """
    Calculate the true anomaly for a given time, period, eccentricity.
    from radvel: https://github.com/California-Planet-Search/radvel
    
    Parameters:
    -----------
    t: array;
        array of times in JD
    tp: float;
        time of periastron, same units as t
    
    per: float;
        orbital period in days
        
    e: float;
        eccentricity

    Returns:
    -------
    array: true anomoly at each time
    """
#	import radvel
    # f in Murray and Dermott p. 27
#    m = 2 * np.pi * (((t - tp) / per) - np.floor((t - tp) / per))
#    eccarr = np.zeros(t.size) + e
#    e1 = radvel.kepler.kepler(m, eccarr)
#    n1 = 1.0 + e
#    n2 = 1.0 - e
#    nu = 2.0 * np.arctan((n1 / n2)**0.5 * np.tan(e1 / 2.0))

#    return nu
    




import numpy as np

def photo_granulation(M,R,Teff):

   """
   Estimate the amplitude and timescale of granulation noise in photometric observations
   as given by Gilliland 2011
   
   Parameters:
   ----------
   
   M: Mass of the star in Solar masses
   
   R: Radius of the star in solar radii
   
   Teff: Effective temperature of the star in Kelvin
   
   Returns
   -------
   
   amp, tau = Amplitude (in ppm) and timescale (in seconds) of the granulation noise in photometry
   
   """
   M, R, Teff= np.array(M), np.array(R), np.array(Teff)
   amp = 75 * M**(-0.5) * R * (Teff/5777.)**0.25  
   
   tau = 220 * M**(-1.) * R**2 * (Teff/5777.)**0.5
   
   return amp, tau
   
def chaplin_exptime(L,Teff,logg):
    """
    Function to compute the optimal exposure time to reduce stellar p-mode oscillation amplitude in the given star to 0.1m/s and 0.09m/s according to Chaplin et al. 2019.
    
    Parameters:
    -----------
    Teff : array_like; size [N]
        An array of stellar effective temperatures in Kelvin.

    logg : array_like; size [N]
        An array of log g's in dex.

    L : array_like; size [N]
        An array of luminosities in units of solar luminosity.

    Returns
    -------
    result : numpy array size [N, 2]
        A 2D array containing tp1 and tpE for each star. tp1 and tpE are exposure times to reach residual amplitudes of 0.1m/s and 0.09m/s to detect earth induced RV. 

    """
    from chaplinfilter import filter
    f = filter(verbose=True)
    results = f(Teff, logg, L)
    return results
    
def rv_precision_degrade(vsini,spec_type):
    """Function to calculate factor by which RV precision of a stellar spectral type degrades due to vsini.
    It compares RV precision at vsini=0km/s to that at *vsini*  in argument of function. It interpolates using quality factor degradation from Table 2 of Bouchy et al.(2001). It fits a 1-D interpolating spline through all the provided data points. 
    
    Parameters:
    ----------
    vsini: vsini of the star
    
    spec_type: Spectral type of the star. Options: "K7V", "K5V", "K2V", "G8V", "F9V", "F5V", "F2V"
    
    Returns
    --------
    Factor by which to multiply the RV precision of the star to get its precision at vsini in argument.
    
    Examples
    --------
    
    For a K2V star whose theoretical RV precision is calculated to be 0.2m/s. This function can be used to estimate the the RV precision at 5km/s.
    
    ::
    
        rv_prec = 0.2
        vsini = 5
        factor=rv_precision_degrade(vsini,K2V)
    
        rv_prec_at_5kms= rv_prec*factor
    
    
    """ 
    import scipy.interpolate as inter    
    
    #table 2 of bouchy et al. (2001)    
    VSINI = [0.,4.,8.,12.,16.,20.]
    
    stars = {'F2V': [14430.0, 9000.0, 4750.0, 2925.0, 2045.0, 1530.0],
             'F5V': [19245.0, 10670.0, 5240.0, 3230.0, 2270.0, 1715.0],
             'F9V': [24450.0, 12685.0, 6105.0, 3785.0, 2690.0, 2045.0],
             'G8V': [30415.0, 14700.0, 7020.0, 4385.0, 3140.0, 2410.0],
             'K2V': [33445.0, 16140.0, 7815.0, 4930.0, 3545.0, 2740.0],
             'K5V': [34940.0, 17080.0, 8440.0, 5380.0, 3885.0, 3020.0],
             'K7V': [31150.0, 15605.0, 8015.0, 5185.0, 3785.0, 2975.0]}    
    
    int_fxn = inter.InterpolatedUnivariateSpline(VSINI,stars[spec_type],k=1)   #
    factor = stars[spec_type][0]/int_fxn(vsini)
    return factor
    



import setuptools
import os, glob
from Planet_tools.__version__ import __version__

here = os.path.abspath(os.path.dirname(__file__))

setuptools.setup(
    name="Planet_tools", # Replace with your own username
    version=__version__,
    author="Babatunde Akinsanmi",
    author_email="tunde.akinsanmi@astro.up.pt",
    description="Package with useful functions for exoplanetary research",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/tundeakins/Planet_tools",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    extras_require = { "dev": [ "pytest >=3.7 "],},
)
